#!/usr/bin/perl -w
# clvenb.pl
# Classement des vétérans ayant au moins "p" participations (5 par défaut)
# Méthode bimbo: on rapporte la place dans le classement au nombre de participants
#                et on fait la moyenne
# Méthode Nakama: on calcule la performance par rapport au CAC40 et on fait la moyenne
#
# Auteur: Jean-Marie Pacquet
#
#
use strict;
use Getopt::Std;
use DBI;
use Net::FTP;
use utf8;
#
my ($sec,$min,$heure,$jour,$mois,$an,$js,$ja,$hiv) = localtime(time);
$an += 1900;
$mois++;
$mois = $mois < 10 ? "0" . $mois : $mois;
$jour = $jour < 10 ? "0" . $jour : $jour;
my $andhui = $an;
my $ann = $an - 2000; # édition du jeu = année sur deux chiffres
print "clvebn.pl $ann\n";
#
###############################################################################
#
# Récupérer les options de la ligne de commande
#
# a=append (ajout au fichier s'il existe)
# b=bimbo
# d=debug (imprime les étapes du calcul)
# n=nakama
# p=nombre minimum de participations
# x=xfer (transfert au site du fichier de classement)
#
our($opt_a, $opt_b, $opt_d, $opt_p, $opt_x);
if (getopts('abdp:x')) {
    if ($opt_b) {
	print "Version bimbo\n";
    } else {
	print "Version nakama\n";
    }
} else {
    die "Usage: clvenb.pl [-[a][b][d][x]] [-p participations]\n";
}
#
my $loginpg = "login";
my $mdppg = "password";
# Ouvrir la base de données
#
my $dbh = DBI->connect('DBI:Pg:dbname=thebull',
		       $loginpg,
		       $mdppg,
		       {
			   AutoCommit => 0
		       }
		       ) or die "Erreur de connexion à la base: " . DBI->errstr;
# Aller chercher la perf de l'indice CAC40 dans la table des résultats
my $rtable = 'resultats';
#
# Aller chercher la liste des portefeuilles des joueurs dans la table des participations
#
my $jtable = 'joueursxx';
my $editions; # nombre d'editions du jeu
my $partmini; # minimum de participations pour le nombre d'editions
#
# préparer la recherche du CAC40 et la recherche des joueurs
#
my $select_reference = $dbh->prepare('SELECT resultat FROM resultats WHERE portif = -1');
my %cac40; # la perf du cac40 par année
my %participants; # nombre de participants par année
my $select_resultat = $dbh->prepare('SELECT resultat FROM resultats WHERE portif = ? and jour = ?');
my $resultat; # résultat
my $date; # date du résultat
my $reveillon;
my $select_participations = $dbh->prepare('SELECT portif, pseudo, an FROM joueursxx WHERE portif IN (SELECT portif FROM joueursxx GROUP BY portif HAVING count(an) >= ?) ORDER BY portif, an DESC');
my @participation; # participation à une édition
my $portif;
my %pseudo; # les pseudos des joueurs
my $numerateur;
my $denominateur;
my %perfxx; # le tableau des performances (perf = $numerateur / $denominateur) de chaque joueur
my %partxx; # tableau du nombre de participations de chaque joueur
my $ctable = 'cotations' . $ann;
my $select_date = $dbh->prepare("SELECT MAX(jour) from $ctable");

die "Impossible de préparer les requêtes\n"
    unless defined $select_reference && defined $select_resultat
    && defined $select_participations && defined $select_date;

# Boucle sur toutes les éditions du JTB
$editions = 0;
for (my $an = 2004; $an <= $andhui; $an++) {
    # Ajuster les paramètres en fonction de l'année
    $date = $an . '-12-31';
    if ($an > 2004) {
	my $an2 = substr($date,2,2);
	substr($rtable,-2,2) = $an2;
	if ($opt_d) {
	    print "Table pour $an = $rtable\n";
	}
	if ($an >= $andhui) {
	    $select_date->execute();
	    if ($select_date->rows == 0) {
		die "Pas de cotation en $andhui\n";
	    }
	    $date = $select_date->fetchrow_array();
	    if ($opt_d) {
		print "Dernier classement le $date ($ja e jour) \n";
	    }
	    $select_date->finish;
	}
    }
    if ($opt_b) {
	# Obtenir le nombre de participants pour cette année
	$select_reference = $dbh->prepare("SELECT MAX(place) FROM $rtable WHERE jour = ?")
	    or die "Impossible de préparer la lecture du nombre de participants\n";
	$select_reference->execute($date);
	if ($select_reference->rows == 0) {
	    die "Pas de résultat pour les participants le $date\n";
	} else {
	    $participants{$an} = $select_reference->fetchrow_array();
	}
	if ($opt_d) {
	    print "Participants $an = $participants{$an} \n";
	}
    } else {
	# Lire la perf de l'indice pour cette année
	$select_reference = $dbh->prepare("SELECT resultat FROM $rtable WHERE portif = -1 AND jour = ?")
	    or die "Impossible de préparer la lecture de la perf de l'indice\n";
	$select_reference->execute($date);
	if ($select_reference->rows == 0) {
	    die "Pas de résultat pour le CAC40 le $date\n";
	} else {
	    $cac40{$an} = $select_reference->fetchrow_array();
	}
	if ($opt_d) {
	    print "CAC40 $an = $cac40{$an} \n";
	}
    }
    # préparation des nouvelles requêtes
    if ($an == 2004) {
	$rtable .= '04';
    }
    $editions++;
}
$select_reference->finish;
$dbh->commit();
$ann = substr($date,2,2); # edition du jeu "11" pour "2011"
my $an1 = substr($date,0,4);
my $mois1 = substr($date,5,2);
my $jour1 = substr($date,8,2);
#
# Boucle de calcul de performance pour chaque portif remplissant les conditions
#
if ($opt_p) {
    $partmini = $opt_p;
} else {
    $partmini = 5;
}
if ($opt_d) {
    print "Participations requises = $partmini pour $editions editions\n";
}
$select_participations->execute($partmini);
$portif = 0;
$an = $andhui;
my $column = "resultat";
if ($opt_b) {
    $column = "place";
}
while ( @participation = $select_participations->fetchrow_array() ) {
    if ($participation[0] != $portif) {
	if ($portif != 0) {
	    # nouveau participant, finir avec le précédent
#	    last; # debug, on s'arrête au premier!
	    $partxx{$portif} = int($denominateur);
	    if (int($denominateur) != $denominateur) {
		$partxx{$portif} .= "+";
	    }
	    # affichage de la performance calculée
	    if (!$opt_b) {
		$perfxx{$portif} = 100 * ($perfxx{$portif} - 1);
	    }
	    if ($opt_d) {
		print "\n$pseudo{$portif} ($partxx{$portif}) Perf=$perfxx{$portif}\n";
	    }
	    # reinitialiser l'année sur la dernière année du classement
	    $an = $andhui;
	}
	# initialiser le portif et le pseudo du participant
	$portif = $participation[0];
	$pseudo{$portif} = "$participation[1]";
	$numerateur = 0;
	$denominateur = 0;
    }
    $an = $participation[2];
    if ($opt_d) {
	print "$an: $participation[1] ($participation[0])\t";
    }
    if ($an == $andhui) {
	$rtable = 'resultats' . substr($date,2,2);
	# Aller chercher le résultat en cours pour ce portif
	$select_resultat = $dbh->prepare("SELECT $column from $rtable where portif = $portif and jour = '$date'");
	$select_resultat->execute();
	if ($select_resultat->rows == 0) {
	    print "Pas de résultat dans $rtable pour portif = $portif et jour = '$date'\n";
	    last;
	} else {
	    $resultat = $select_resultat->fetchrow_array();
	}
	# Initialiser la performance au pro rata de l'année en cours
	$denominateur = $ja / 365;
	if ($opt_b) {
	    $numerateur = ($ja * $resultat) / (365 * $participants{$an});
	    $perfxx{$portif} = 100 * $numerateur;
	} else {
	    $numerateur = 1 + (($resultat - $cac40{$an}) / (100 + $cac40{$an}));
	    $perfxx{$portif} = $numerateur;
#	    $numerateur *= $denominateur; # moyenne arithmétique pondérée
	    $numerateur = $denominateur * log($numerateur); # moyenne géométrique pondérée
	}
    } else {
	$reveillon = $an . '-12-31';
	if ($an > 2004) {
	    $rtable = 'resultats' . substr($reveillon,2,2);
	} else {
	    $rtable = 'resultats';
	}
	# Aller chercher le résultat de cette année là pour ce portif
	$select_resultat = $dbh->prepare("SELECT $column FROM $rtable WHERE portif = $portif AND jour = '$reveillon'");
	$select_resultat->execute();
	if ($select_resultat->rows == 0) {
	    print "Pas de résultat dans $rtable pour portif = $portif et jour = '$reveillon'\n";
	    last;
	} else {
	    $resultat = $select_resultat->fetchrow_array();
	}
	if ($opt_b) {
	    $numerateur += $resultat / $participants{$an};
	} else {
#	    $numerateur += ($resultat - $cac40{$an}) / (100 + $cac40{$an}); # moyenne arithmétique pondérée
	    $numerateur += log(1 + (($resultat - $cac40{$an}) / (100 + $cac40{$an}))); # moyenne géométrique pondérée
	}
	$denominateur++;
	if ($opt_b) {
	    $perfxx{$portif} = (100 * $numerateur)/ $denominateur;
	} else {
#	    $perfxx{$portif} = (100 * $numerateur)/ $denominateur; # moyenne arithmétique pondérée
	    $perfxx{$portif} = exp($numerateur / $denominateur);
	}
    }
    if ($opt_d) {
	print "Perf=$perfxx{$portif}\n";
    }
    $an--;
}
if ($portif != 0) {
    if (!$opt_b) {
	$perfxx{$portif} = 100 * ($perfxx{$portif} - 1);
    }
    $partxx{$portif} = int($denominateur);
    if (int($denominateur) != $denominateur) {
	$partxx{$portif} .= "+";
    }
    # affichage de la performance calculée
    if ($opt_d) {
	print "\n$pseudo{$portif} ($partxx{$portif}) Perf=$perfxx{$portif}\n";
    }
}
$select_resultat->finish;
$select_participations->finish;
#
# tri par ordre décroissant de la performance
#
my @classement;
# classement final
if ($opt_b) {
    @classement = sort { $perfxx{$a} <=> $perfxx{$b} } keys %perfxx;
} else {
    @classement = sort { $perfxx{$b} <=> $perfxx{$a} } keys %perfxx;
}
# affichage du classement (dans un fichier en html)
my $fichier_cumul = "cumul". substr($an1,2,2) . $mois1 . $jour1 . ".html";
open(CUMUL, ($opt_a ? ">>" : ">") . $fichier_cumul);
if ($opt_b) {
    print CUMUL "<h3 class=t6>Classement Bimbo (Moyenne de la place rapportée au nombre de participants pour chaque participation)</h3>\n";
} else {
    print CUMUL "<h3 class=t6>Classement par rapport à l'indice CAC40 (Moyenne géométrique de l'écart à l'indice pour chaque participation)</h3>\n";
}
print CUMUL "<h4>Au minimum $partmini participations</h4>";
print CUMUL "<h5 class=t6>(Fait le " . $jour . "/" . $mois . "/" . $andhui . " à ";
print CUMUL $heure . ":" . $min . ":" . $sec . ")</h5>\n";
print CUMUL "<table border=0 cellpadding=1 cellspacing=1 width=100%><tbody>";
my $place = 0;
foreach $portif (@classement) {
    if ($portif < 0) {
	print CUMUL "<tr class=t1><td>&nbsp;</td>";
    } else {
	$place += 1;
	print CUMUL "<tr class=t1><td>" . $place . "</td>";
    }
    print CUMUL "<td colspan=2>" . $pseudo{$portif} . "</td><td>$partxx{$portif}</td>";
    if ($perfxx{$portif} < 0) {
	print CUMUL "<td class=t3><b>";
    } else {
	print CUMUL "<td class=t2><b>";
    }
    print CUMUL sprintf("%+.2f",$perfxx{$portif}) . "</b></td>";
    print CUMUL "</tr>\n";
}
print CUMUL "</tbody></table>\n<br>\n";
close(CUMUL);
#
# fin des boucles, on ferme!
#
$dbh->disconnect;
#
# Transfert du fichier html vers le serveur (option -x)
#
if ($opt_x) {
    my $ftp = new Net::FTP("ftp.server.ext", Debug => 0, Passive => 1);
    $ftp->login("login","password");
    $ftp->cwd("/thebull/20" . $ann);
    $ftp->put($fichier_cumul);
    $ftp->quit;
}
# c'est fini!
exit(0);


